/*
 * main.c
 */

#include <msp430.h>
#include "main.h"
#if defined(__MSP430G2231__)
#include "USI_SPI.h"
#elif defined(__MSP430G2553__)
#include "USCI_SPI.h"
#endif
#include "Button.h"
#include "PGA450Comm.h"


unsigned int sysTick;
TIMER16 timer16[TIMER16_MAX];


void kickDog(void)
{
    WDTCTL = (WDTPW+WDTCNTCL+WDTSSEL);

}


void freezeDog(void)
{
    WDTCTL = (WDTPW+WDTHOLD+WDTSSEL);
}


void main(void)
{
    kickDog();
	portInit();
	sysClkInit();
	timerAInit();
    #if defined(__MSP430G2231__)
	  usiSPIInit();
    #elif defined(__MSP430G2553__)
	  ucsiSPIInit();
    #endif
	pga450Init();

	__enable_interrupt();

	for(;;)
	{
	    kickDog();
		sysTimerTask();
		buttonTask();
		pga450ProgTask();
		//if(timer16[RUN_LED_PER] == 0)
		//{
		//    timer16[RUN_LED_PER] = period(500);
		//    runLEDPort ^= RUN_LED_PIN;
		//}

		//P2OUT |= 0x20;
	}
}


// Initialize port
void portInit(void)
{
	// P1.0 - VPWR_EN
	// P1.1 - LED D6
	// P1.2 - CS5
	// P1.3 - LED D5
	// P1.4 - LED D4
	// P1.5 - SCLK
	// P1.6 - MISO
	// P1.7 - MOSI
	P1OUT = 0x05;  	//0b00000101	 //0b00011010
	P1DIR = 0xBF;	//0b10111111	 //0b10110111
	P1REN = 0x00; 	//0b00000000	 //0b00001000
	P1SEL = 0xE0; //0b11100000
    #if defined(__MSP430G2553__)
	  P1SEL2 = 0xE0; //0b11100000
    #endif
	P1IFG = 0;
	P1IES = 0;
	P1IE = 0;

	// P2.0 - CS1
	// P2.1 - LED D3
	// P2.2 - CS2
	// P2.3 - Start jumper
	// P2.4 - CS3
	// P2.5 - LED D7
	// P2.6 - CS4
	// P2.7 - VOPT_EN
	P2OUT = 0xDD;	//0b11011101		// Output high/HiZ
    P2DIR = 0xF7;	//0b11110111	 // Output/Input
    P2REN = 0x00; 	// Pull resistor disabled
    P2SEL = 0;
    #if defined(__MSP430G2553__)
      P2SEL2 = 0;
    #endif
    P2IFG = 0;
    P2IES = 0;
    P2IE = 0;

#if defined(__MSP430G2553__)
	// P3.0 -
	// P3.1 -
	// P3.2 -
	// P3.3 -
	// P3.4 -
	// P3.5 -
	// P3.6 -
	// P3.7 -
    P3OUT = 0x00; // Pull down
    P3DIR = 0; // Input
    P3REN = 0xFF; // Pull resistor enabled
    P3SEL = 0;
    P3SEL2 = 0;
#endif
}


// Initialize MCU clock
// Clock source: internal DCO
// Frequency: 16MHz (calibrated)
void sysClkInit(void)
{
	DCOCTL = 0;
	
	BCSCTL2 = 0;
	BCSCTL3 = 0;

	#if defined(__MSP430G2553__)
      BCSCTL1 = CALBC1_1MHZ|0x00; // Set ACLK = DCOCLK/1
      DCOCTL = CALDCO_1MHZ;
      runLEDPort |= RUN_LED_PIN;
      unsigned int i = 50000;
      while(i--);
      runLEDPort &= ~RUN_LED_PIN;
      BCSCTL1 = CALBC1_16MHZ|0x30; // Set ACLK = DCOCLK/8
      DCOCTL = CALDCO_16MHZ;
    #endif
    #if defined(__MSP430G2231__)
	  BCSCTL1 = CALBC1_1MHZ|0x30; // Set ACLK = DCOCLK/8
	  DCOCTL = CALDCO_1MHZ;
    #endif
}


// Initialize timer A
// Clock source: MCLK/1 = 16MHz/1 = 16MHz
// Continuous mode
//
// Timer A Channel 0
// Compare mode, periodic interrupt @ 0.001ms
void timerAInit(void)
{
	//TACTL = 0b0000001000100000;
	TACTL = 0x0220;

	// Channel 0 is system tick counter
	//TACCTL0 = 0b0000000000010000;
	TACCTL0 = 0x0010;
	TACCR0 = TAC0_PERIOD;

	// Channel 1 is manual delay counter
	TACCTL1 = 0x0000;
}


// Delay in microsecond
// Very short delay may lead to inaccurate result
// Better freeze watchdog before use
void delay_us(unsigned int uSec)
{
    TACCR1 = TAR+uSec;
    TACCTL1 &= (~CCIFG);

    while(!(TACCTL1 & CCIFG));
}


#if defined(__MSP430G2553__)
#pragma vector=TIMER0_A0_VECTOR
#else
#pragma vector=TIMERA0_VECTOR
#endif
__interrupt void TAC0_ISR(void)
{
	TACCR0 += TAC0_PERIOD;
	sysTick++;
	//runLEDPort ^= RUN_LED_PIN;
}


void sysTimerTask(void)
{
	unsigned int tick;
	TIMER16* pTimer16;

	if(sysTick > 0)
	{
		TACCTL0 &= (~CCIE);
		tick = sysTick;
		sysTick = 0;
		TACCTL0 |= CCIE;

		pTimer16 = timer16;
		while(pTimer16 < &timer16[TIMER16_MAX])
		{
		    if(*pTimer16 > 0)
		    {
		        if(*pTimer16 < tick)
		            *pTimer16 = 0;
		        else
		            *pTimer16 -= tick;
		    }
		    pTimer16++;
		}
	}
}


#if defined(__MSP430G2553__)
#pragma vector=NMI_VECTOR,TIMER1_A0_VECTOR,TIMER1_A1_VECTOR,COMPARATORA_VECTOR,WDT_VECTOR,TIMER0_A1_VECTOR,USCIAB0RX_VECTOR,USCIAB0TX_VECTOR,ADC10_VECTOR,PORT2_VECTOR,PORT1_VECTOR
#else
#pragma vector=NMI_VECTOR,TIMERA1_VECTOR,WDT_VECTOR,USI_VECTOR,ADC10_VECTOR,PORT2_VECTOR,PORT1_VECTOR
#endif
__interrupt void nop_ISR(void)
{

}
